import safelz4._block as _block

__all__ = [
    "compress",
    "compress_str_utf8",
    "compress_into",
    "compress_utf8_prepend_size",
    "compress_prepend_size",
    "decompress",
    "decompress_into",
    "decompress_size_prepended",
    "decompress_with_dict",
    "get_maximum_output_size",
]

get_maximum_output_size = _block.get_maximum_output_size


def compress(input: bytes) -> bytes:
    """
    Compress all bytes of input.

    Args:
        input (`bytes`):
            An arbitrary byte buffer to be compressed.

    Returns:
        `bytes`: lz4 compressed block.
    """
    return _block.compress(input)


def compress_str_utf8(input: str) -> bytes:
    """
    Compress str that is utf-8 encodable into a block.

    Args:
        input (`str`)
            An arbitrary string buffer.

    Returns:
        `Tuple[int, bytes]` : size of the compressed, and the block.
    """
    try:
        buffer = input.encode("utf-8")
    except UnicodeEncodeError:
        raise UnicodeEncodeError(
            f"input {input[:min(len(input), 5)]:>5} is not utf-8 encodable."
        )
    return _block.compress(buffer)


def compress_into(input: bytes, output: bytearray) -> int:
    """
    Compress all bytes of input into the output array assuming size its known.

    Args:
        input (`bytes`):
            fixed set of bytes to be compressed.
        output (`bytearray`):
            Mutable buffer to hold decompressed bytes.

    Returns:
        `int` : size of the compressed bytes
    """
    return _block.compress_into(input, output)


def compress_utf8_prepend_size(input: str) -> bytes:
    """
    Compress all utf-8 compatible strings of input into output.
    The uncompressed size will be prepended as a little endian u32.

    Args:
        input (`str` that is uft-8 compatible):
            fixed set of `str` that is utf-8 encodable.

    Returns:
        `bytes`: compressed `block` format
    """
    try:
        buffer = input.encode("utf-8")
        return bytes(_block.compress_prepend_size(buffer))
    except UnicodeEncodeError:
        raise UnicodeEncodeError(
            f"input {input[:min(len(input), 5)]:>5} is not utf-8 encodable."
        )


def compress_prepend_size(input: bytes) -> bytes:
    """
    Compress the input bytes using LZ4 and prepend the original
    size as a little-endian u32. This is compatible with
    `decompress_size_prepended`.

    Args
        input : (`bytes`)
            fixed set of bytes to be compressed.

    Returns
        `bytes`:
            Compressed data with the uncompressed size prepended.
    """
    return _block.compress_prepend_size(input)


def decompress(input: bytes, min_size: int) -> bytes:
    """
    Decompress the input block bytes.

    Args:
        input (`bytes`)
            fixed set of bytes to be decompressed
        min_size (`int`):
            minimum possible size of uncompressed bytes

    Returns:
        `bytes`: decompressed repersentation of the compressed bytes.
    """
    return _block.decompress(input, min_size)


def decompress_into(input: bytes, output: bytearray) -> int:
    """
    Decompress input bytes into the provided output buffer.
    The output buffer must be preallocated with enough space
    for the uncompressed data.

    Args:
        input (`bytes`):
            Fixed set of bytes to be decompressed.
        output (`bytearray`):
            Mutable buffer to hold decompressed bytes.

    Returns:
        `int`: Number of bytes written to the output buffer.
    """
    return _block.decompress_into(input, output)


def decompress_size_prepended(input: bytes) -> bytes:
    """
    Decompress lz4 compressed block byte file format

    Args:
        input : (`bytes`)
            fixed set of bytes to be compressed.

    Returns:
        `bytes`: decompressed repersentation of the compressed bytes.
    """
    return _block.decompress_size_prepended(input)


def decompress_with_dict(input: bytes, ext_dict: bytes) -> bytes:
    """
    Decompress input bytes using a user-provided dictionary of bytes.

    Args:
        input (`bytes`):
            fixed set of bytes to be decompressed.
        ext_dict (`bytes`):
            Dictionary used for decompression.

    Returns:
        `bytes`: Decompressed data.
    """
    _block.decompress_with_dict(input, ext_dict)



def compress(input: bytes) -> bytes:
    """
    Compress the input bytes using LZ4.

    Args:
        input (`bytes`):
            fixed set of bytes to be compressed.

    Returns:
        (`bytes`): Compressed LZ4 block format.
    """
    ...

def compress_prepend_size(input: bytes) -> bytes:
    """
    Compress the input bytes using LZ4 and prepend the original size as
    a little-endian u32.

    This is compatible with `decompress_size_prepended`.

    Args:
        input (`bytes`):
            fixed set of bytes to be compressed.

    Returns:
        (`bytes`): Compressed LZ4 block data with the uncompressed size prepended.
    """
    ...

def compress_into(input: bytes, output: bytearray) -> int:
    """
    Compress the input bytes into the provided output buffer. The output buffer
    must be preallocated with a size obtained from `get_maximum_output_size`.

    Args:
        input (`bytes`):
            fixed set of bytes to be compressed.
        output (`bytearray`):
            Mutable buffer to hold combessed bytes.

    Returns:
        (`int`): Number of bytes written to the output buffer.
    """
    ...

def compress_with_dict(input: bytes, dictionary: bytes) -> bytes:
    """
    Compress the input bytes using a user-provided dictionary.

    Args:
        input (`bytes`):
            fixed set of bytes to be compressed.
        dictionary (`bytes`):
            Dictionary used for compression.

    Returns:
        (`bytes`): fixed set of bytes to be decompressed.
    """
    ...

def decompress(input: bytes, min_size: int) -> int:
    """
    Decompress the input block bytes.

    Args:
        input (`bytes`)
            fixed set of bytes to be decompressed
        min_size (`int`):
            minimum possible size of uncompressed bytes

    Returns:
        (`bytes`): decompressed repersentation of the compressed bytes.
    """
    ...

def decompress_into(input: bytes, output: bytearray) -> int:
    """
    Decompress input bytes into the provided output buffer.
    The output buffer must be preallocated with enough space
    for the uncompressed data.

    Args:
        input (`bytes`):
            Fixed set of bytes to be decompressed.
        output (`bytearray`):
            Mutable buffer to hold decompressed bytes.

    Returns:
        (`int`): Number of bytes written to the output buffer.
    """
    ...

def decompress_size_prepended(input: bytes) -> bytes:
    """
    Decompress input bytes that were compressed with the original
    size prepended. Compatible with `compress_prepend_size`.

    Args:
        input (`bytes`):
            fixed set of bytes to be decompressed

    Returns:
        (`bytes`): Decompressed data.
    """
    ...

def decompress_with_dict(input: bytes, ext_dict: bytes) -> bytes:
    """
    Decompress input bytes using a user-provided dictionary of
    bytes.

    Args:
        input (`bytes`):
            fixed set of bytes to be decompressed.
        ext_dict (`bytes`):
            Dictionary used for decompression.

    Returns:
        (`bytes`): Decompressed data.
    """
    ...

def get_maximum_output_size(input_len: int) -> int:
    """
    Obtain the maximum output size of the block

    Args:
        input_len (`int`):
            length of the bytes we need to allocate to compress
            into fixed buffer.
    Returns:
        (`int`):
            maximum possible size of the output buffer needs to be."""
    ...



from safelz4._safelz4_rs import _frame

__all__ = ["BlockMode", "BlockSize", "FrameInfo"]

BlockMode = _frame.BlockMode
BlockSize = _frame.BlockSize
FrameInfo = _frame.FrameInfo

open_frame = _frame.open_frame

compress = _frame.compress
compress_file = _frame.compress_file
compress_file_with_info = _frame.compress_file_with_info
compress_with_info = _frame.compress_with_info

decompress = _frame.decompress
decompress_file = _frame.decompress_file

FLG_RESERVED_MASK = _frame.FLG_RESERVED_MASK
FLG_VERSION_MASK = _frame.FLG_VERSION_MASK
FLG_SUPPORTED_VERSION_BITS = _frame.FLG_SUPPORTED_VERSION_BITS

FLG_INDEPENDENT_BLOCKS = _frame.FLG_INDEPENDENT_BLOCKS
FLG_BLOCK_CHECKSUMS = _frame.FLG_BLOCK_CHECKSUMS
FLG_CONTENT_SIZE = _frame.FLG_CONTENT_SIZE
FLG_CONTENT_CHECKSUM = _frame.FLG_CONTENT_CHECKSUM
FLG_DICTIONARY_ID = _frame.FLG_DICTIONARY_ID

BD_RESERVED_MASK = _frame.BD_RESERVED_MASK
BD_BLOCK_SIZE_MASK = _frame.BD_BLOCK_SIZE_MASK
BD_BLOCK_SIZE_MASK_RSHIFT = _frame.BD_BLOCK_SIZE_MASK_RSHIFT

LZ4F_MAGIC_NUMBER = _frame.LZ4F_MAGIC_NUMBER
LZ4F_LEGACY_MAGIC_NUMBER = _frame.LZ4F_LEGACY_MAGIC_NUMBER

MAGIC_NUMBER_SIZE = _frame.MAGIC_NUMBER_SIZE
MIN_FRAME_INFO_SIZE = _frame.MIN_FRAME_INFO_SIZE
MAX_FRAME_INFO_SIZE = _frame.MAX_FRAME_INFO_SIZE
BLOCK_INFO_SIZE = _frame.BLOCK_INFO_SIZE



from ._safelz4_rs import __version__, error
import safelz4.block as block
import safelz4.frame as frame
from safelz4.frame import compress, decompress, decompress_file, compress_file
from safelz4._frame import BlockMode, BlockSize, FrameInfo, open_frame

LZ4Exception = error.LZ4Exception

__all__ = [
    "__version__",
    "block",
    "frame",
    "BlockMode",
    "BlockSize",
    "FrameInfo",
    "LZ4Exception",
    "compress",
    "decompress",
    "decompress_file",
    "compress_file",
    "open_frame",
]



import os
from typing import Optional, Union, Any
from typing_extensions import Self

from enum import IntEnum, Enum

class BlockMode(Enum):
    """
    Block mode for frame compression.

    Attributes:
        Independent: Independent block mode.
        Linked: Linked block mode.
    """

    Independent = "Independent"
    Linked = "Linked"

class BlockSize(IntEnum):
    """
    Block size for frame compression.

    Attributes:
        Auto: Will detect optimal frame size based on the size of the first
        write call.
        Max64KB: The default block size (64KB).
        Max256KB: 256KB block size.
        Max1MB: 1MB block size.
        Max4MB: 4MB block size.
        Max8MB: 8MB block size.
    """

    Auto = 0
    Max64KB = 4
    Max256KB = 5
    Max1MB = 6
    Max4MB = 7
    Max8MB = 8

class FrameInfo:
    """
    Information about a compression frame.

    Attributes:
        content_size: If set, includes the total uncompressed size of data in
                      the frame.
        block_size: The maximum uncompressed size of each data block.
        block_mode: The block mode.
        block_checksums: If set, includes a checksum for each data block in the
        frame.
        content_checksum: If set, includes a content checksum to verify that the
          full frame contents have been decoded correctly.
        legacy_frame: If set, use the legacy frame format.
    """

    content_size: Optional[int]
    block_size: BlockSize
    block_mode: BlockMode
    block_checksums: bool
    dict_id: Optional[int]
    content_checksum: bool
    legacy_frame: bool

    def __new__(
        self,
        block_size: BlockSize,
        block_mode: BlockMode,
        block_checksums: Optional[bool] = None,
        dict_id: Optional[int] = None,
        content_checksum: Optional[bool] = None,
        content_size: Optional[int] = None,
        legacy_frame: Optional[bool] = None,
    ) -> Self: ...
    @staticmethod
    def default() -> Self:
        """
        build a default `FrameInfo` class.

        Returns:
            (`FrameInfo`): default object.
        """
        ...
    @staticmethod
    def read_header_info(input: bytes) -> Self:
        """Read header info to construct header."""
        ...
    def read_header_size(input: bytes) -> Self:
        """Read the size of the header info"""
        ...
    @property
    def block_checksums(self) -> bool: ...
    @block_checksums.setter
    def block_checksums(self, value: bool) -> None: ...
    @property
    def block_size(self) -> BlockSize: ...
    @block_size.setter
    def block_size(self, value: BlockSize) -> None: ...
    @property
    def block_mode(self) -> BlockMode: ...
    @property
    def content_size(self) -> Optional[int]: ...
    @content_size.setter
    def content_size(self, value: int) -> None: ...
    @property
    def content_sum(self) -> bool: ...
    @property
    def content_checksum(self) -> bool: ...
    @content_checksum.setter
    def content_checksum(self, value: bool) -> None: ...
    @property
    def legacy_frame(self) -> bool: ...
    @legacy_frame.setter
    def legacy_frame(self, value: bool) -> None: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...

class open_frame:
    """
    Context manager that allows us to decompresses a buffer of bytes using thex
    LZ4 frame format.

    Example:
    ```
    output = None
    with open_frame("datafile") as f:
        output = f.decompress()

    print(output)
    ```
    """

    def __new__(self, filename: Union[str, os.PathLike]) -> "open_frame":
        """Initialize the open_frame context manager with a filename."""
        ...
    def info(self) -> FrameInfo:
        """
        Return the frameinfo of the compression file.
        Returns:
            `FrameInfo`:
                The freeform FrameInfo.
        """
        ...
    def decompress(self) -> bytes:
        """
        Decompress the whole frame file.
        Returns:
            (`bytes`):
                The decompressed (original) representation of the bytes
                within the file.
        """
        ...
    def content_size(self) -> Optional[int]:
        """Return the content size if available in the frame header."""
        ...
    def __enter__(self) -> "open_frame":
        """Enter the context manager and return self."""
        ...
    def __exit__(
        self,
        exc_type: Optional[type],
        exc_val: Optional[Exception],
        exc_tb: Optional[Any],
    ) -> None:
        """Exit the context manager and clean up resources."""
        ...

def decompress(input: bytes) -> bytes:
    """
    Decompresses a buffer of bytes using thex LZ4 frame format.

    Args:
        input (`bytes`):
            A byte containing LZ4-compressed data (in frame format).
            Typically obtained from a prior call to an `compress` or read from
            a compressed file `compress_file`.

    Returns:
        (`bytes`):
            The decompressed (original) representation of the input bytes.
    """
    ...

def decompress_file(filename: Union[os.PathLike, str]) -> bytes:
    """
    Decompresses a buffer of bytes into a file using thex LZ4 frame format.

    Args:
        filename (`str` or `os.PathLike`):
            The filename we are loading from.

    Returns:
        (`bytes`):
            The decompressed (original) representation of the input bytes.
    """
    ...

def compress(input: bytes) -> bytes:
    """
    Compresses a buffer of LZ4-compressed bytes using the LZ4 frame format.

    Args:
        input (`bytes`):
            An arbitrary byte buffer to be compressed.
    Returns:
        `bytes`:
             The LZ4 frame-compressed representation of the input bytes.
    """
    ...

def compress_file(filename: Union[os.PathLike, str], input: bytes) -> None:
    """
    Compresses a buffer of bytes into a file using using the LZ4 frame format.

    Args:
        filename (`str` or `os.PathLike`):
            The filename we are saving into.
        input (`bytes`):
            un-compressed representation of the input bytes.

    Returns:
        (`None`)
    """
    ...

def compress_file_with_info(
    filename: Union[os.PathLike, str],
    input: bytes,
    info: Optional[FrameInfo] = None,
) -> None:
    """
    Compresses a buffer of bytes into a file using using the LZ4 frame format,
    with more control on Frame.

    Args:
        filename (`str`, or `os.PathLike`):
            The filename we are saving into.
        input (`bytes`):
            fixed set of bytes to be compressed.
        info (`FrameInfo, *optional*, defaults to `None``):
            The metadata for de/compressing with lz4 frame format.

    Returns:
        (`None`)
    """
    ...

def compress_with_info(
    input: bytes,
    info: Optional[FrameInfo] = None,
) -> None:
    """
    Compresses a buffer of bytes into byte buffer using using the LZ4 frame
    format, with more control on Frame.

    Args:
        input (`bytes`):
            fixed set of bytes to be compressed.
        info (`FrameInfo, *optional*, defaults to `None``):
            The metadata for de/compressing with lz4 frame format.

    Returns:
        (`bytes`):
            The LZ4 frame-compressed representation of the input bytes.
    """
    ...



from safelz4._safelz4_rs import _block

compress = _block.compress
compress_prepend_size = _block.compress_prepend_size
compress_into = _block.compress_into
compress_with_dict = _block.compress_with_dict

decompress = _block.decompress
decompress_into = _block.decompress_into
decompress_size_prepended = _block.decompress_size_prepended
decompress_with_dict = _block.decompress_with_dict

get_maximum_output_size = _block.get_maximum_output_size



import os

from typing import Union, Optional
import safelz4._frame as _frame
from safelz4._frame import FrameInfo, BlockMode, BlockSize

__all__ = [
    "FrameInfo",
    "BlockMode",
    "BlockSize",
    "decompress",
    "compress",
    "decompress_file",
    "compress_file",
    "compress_file_with_info",
    "compress_with_info",
]


def decompress(input: bytes) -> bytes:
    """
    Decompresses a buffer of bytes using thex LZ4 frame format.

    Args:
        input (`bytes`):
            A byte containing LZ4-compressed data (in frame format).
            Typically obtained from a prior call to an `compress` or read from
            a compressed file `compress_file`.

    Returns:
        (`bytes`):
            The decompressed (original) representation of the input bytes.

    Example:

    ```python
    from safelz4 import decompress

    output = None
    with open("datafile.lz4", "r")  as file:
        buffer = file.read(-1).encode("utf-8")
        output = decompress(buffer)
    ```
    """
    return _frame.decompress(input)


def decompress_file(filename: Union[os.PathLike, str]) -> bytes:
    """
    Decompresses a buffer of bytes into a file using thex LZ4 frame format.

    Args:
        filename (`str` or `os.PathLike`):
            The filename we are loading from.

    Returns:
        (`bytes`):
            The decompressed (original) representation of the input bytes.

    Example:

    ```python
    from safelz4 import decompress

    output = decompress("datafile.lz4")
    ```

    """
    return _frame.decompress_file(filename)


def compress(input: bytes) -> bytes:
    """
    Compresses a buffer of LZ4-compressed bytes using the LZ4 frame format.

    Args:
        input (`bytes`):
            An arbitrary byte buffer to be compressed.
    Returns:
        (`bytes`):
             The LZ4 frame-compressed representation of the input bytes.

    Example:
    ```python
    from safelz4.frame import compress

    buffer = None
    with open("datafile.txt", "r") as file:
        output = file.read(-1).encode("utf-8")
        buffer = compress(output)

    # ...
    ```
    """
    return _frame.compress(input)


def compress_file(filename: Union[os.PathLike, str], input: bytes) -> None:
    """
    Compresses a buffer of bytes into a file using using the LZ4 frame format.

    Args:
        filename (`str` or `os.PathLike`):
            The filename we are saving into.
        input (`bytes`):
            un-compressed representation of the input bytes.

    Example:
    ```python
    from safelz4.frame import compress

    with open("datafile.txt", "r") as file:
        buffer = file.read(-1).encode("utf-8")
        compress_file("datafile.lz4", buf_f)

    ```

    Returns:
        (`None`)
    """
    return _frame.compress_file(filename, input)


def compress_file_with_info(
    filename: Union[os.PathLike, str],
    input: bytes,
    info: Optional[FrameInfo] = None,
) -> None:
    """
    Compresses a buffer of bytes into a file using using the LZ4 frame format,
    with more control on Block Linkage.

    Args:
        filename (`str`, or `os.PathLike`):
            The filename we are saving into.
        input (`bytes`):
            fixed set of bytes to be compressed.
        info (`FrameInfo, *optional*, defaults to `None``):
            The metadata for de/compressing with lz4 frame format.

    Returns:
        (`None`)
    """
    return _frame.compress_file_with_info(filename, input, info)


def compress_with_info(
    input: bytes,
    info: Optional[FrameInfo] = None,
) -> None:
    """
    Compresses a buffer of bytes into byte buffer using using
    the LZ4 frame format, with more control on Frame.

    Args:
        input (`bytes`):
            fixed set of bytes to be compressed.
        info (`FrameInfo, *optional*, defaults to `None``):
            The metadata for de/compressing with lz4 frame format.

    Returns:
        (`bytes`):
            The LZ4 frame-compressed representation of the input bytes.
    """
    return _frame.compress_with_info(input, info)
